/*
 * DDS MICRO-C Command Coordinator
 *
 * This program integrates the various commands required to compile a
 * program into a single command.
 *
 * The following assemblers and linkers have been reported to work OK with
 * MICRO-C, using the indicated command line options:
 *
 * Vendor    Name  Versions(s)     Options
 * ---------------------------------------
 * Arrowsoft ASM   1.00d (PD)		/ml
 * Arrowsoft ASM   2.00c (PD)		/s/ml
 * Microsoft MASM  3.0				/ml
 * Microsoft MASM  4.0, 5.1			/t/ml
 * Borland   TASM  1.0, 2.01, 3.0	/t/ml
 * Watcom    WASM  1.0              /q
 * Troendle  VAL   None (PD)		/nci [/COM] *
 * Microsoft LINK  3.x, 5.x			/noi
 * Borland   TLINK 2.0, 3.01, 4.01	/c
 * Watcom    WLINK 10.0             form DOS [COM]
 *
 ** Original VAL did not work correctly for TINY model.
 *  Fixed version is available from our BBS.
 *
 * If your assembler and linker are not compatible with the command lines
 * hard coded in this program, do the following:
 *
 * - Modify CC.C, LC.BAT and PC86.IDE to use acceptable commands
 * - Compile CC.C using DDSIDE
 *         - or -
 * - Compile CC.C by running MCP, MCC, Assembler and LC commands:
 *       MCP CC.C CC.C1 l=.
 *       MCC CC.C1 CC.ASM
 *       Assemble with your assembler to get CC.OBJ
 *       LC CC
 * - Test CC.COM by re-compiling itself: CC CC -fop
 *
 * Where possible, the assembler and linker should be run with appropriate
 * options to cause them to be CASE SENSITIVE, and QUIET in operation (no
 * output messages). If either your assembler OR linker does not support
 * case sensitive operation, run BOTH of them in case insensitive mode,
 * and be sure not to use any global symbols which differ only in case.
 *
 * Copyright 1990-1995 Dave Dunfield
 * All rights reserved.
 *
 * Compile command: cc cc -fop
 */
#include <stdio.h>

#define	NOFILE	2		/* EXEC return code for file not found */
#define	NOPATH	3		/* EXEC return code for path not found */

static char mcdir[65], temp[65], ofile[65], tail[150], mcparm[80];
static char oasm = -1, opt = 0, pre = 0, tiny = -1, link = -1, verb = -1,
	del = -1, com = 0, fold = 0, *fnptr, *mptr = &mcparm;

/*
 * Main program, process options & invoke appropriate commands
 */
main(int argc, char *argv[])
{
	int i;
	char ifile[65], *ptr, c;

	/* Get default directories from environment */
	if(!getenv("MCDIR", mcdir)) {	/* Get MICRO-C directory */
		message("Environment variable MCDIR is not set!\n\n");
		strcpy(mcdir, "C:\\MC"); }
	if(!getenv("MCTMP", temp)) {	/* Get temporary directory */
		if(getenv("TEMP", temp)) {
			if(temp[(i = strlen(temp))-1] != '\\') {
				temp[i] = '\\';
				temp[i+1] = 0; } } }

/* parse for command line options. */
	for(i=2; i < argc; ++i) {
		if(*(ptr = argv[i]) == '-') {		/* Enable switch */
			while(*++ptr) {
				switch(toupper(*ptr)) {
					case 'A' : oasm = 0;	continue;
					case 'C' : com = -1;	continue;
					case 'F' : fold = -1;	continue;
					case 'K' : del = 0;		continue;
					case 'L' : link = 0;	continue;
					case 'O' : opt = -1;	continue;
					case 'P' : pre = -1;	continue;
					case 'Q' : verb = 0;	continue;
					case 'S' : tiny = 0;	continue; }
				goto badopt; }
			continue; }

		if(*(ptr+1) == '=') switch(toupper(*ptr)) {
			case 'H' : strcpy(mcdir, ptr+2);	continue;
			case 'T' : strcpy(temp, ptr+2);		continue; }

		*mptr++ = ' ';
		c = 0;
		while(*mptr++ = *ptr++) {
			if(*ptr == '=')
				c = pre; }
		if(c)
			continue;

	badopt:
		fprintf(stderr,"Invalid option: %s\n", argv[i]);
		exit(-1); }

	message("DDS MICRO-C PC86 Compiler v3.14\n");

	if(argc < 2)
		abort("\nUse: CC <name> [-acfklopqs h=homedir t=tmpdir] [symbol=value]\n\nCopyright 1990-1995 Dave Dunfield\nAll rights reserved.\n");

	/* Parse filename & extension from passed path etc. */
	fnptr = ptr = argv[1];
	while(c = *ptr) {
		if(c == '.')
			goto noext;
		++ptr;
		if((c == ':') || (c == '\\'))
			fnptr = ptr; }
	strcpy(ptr, ".C");
noext:
	strcpy(ifile, argv[1]);
	message(fnptr);
	message(": ");
	*mptr = *ptr = 0;

	/* Pre-process to source file */
	if(pre) {
		message("Preprocess... ");
		sprintf(ofile,"%s%s.CP", temp, fnptr);
		sprintf(tail,"%s %s l=%s -q -l%s", ifile, ofile, mcdir, mcparm);
		docmd("MCP.EXE");
		strcpy(ifile, ofile); }

	/* Compile to assembly language */
	message("Compile... ");
	sprintf(ofile,"%s%s.%s", (oasm||opt) ? temp : "", fnptr,
		opt ? "CO" : "ASM");
	sprintf(tail,"%s %s -q%s%s%s", ifile, ofile, pre ? " -l" : "",
		com ? " -c" : "", fold ? " -f" : "");
	docmd("MCC.EXE");
	if(pre)
		erase(ifile);
	strcpy(ifile, ofile);

	/* Optimize the assembly language */
	if(opt) {
		message("Optimize... ");
		sprintf(ofile,"%s%s.ASM", oasm ? temp : "", fnptr);
		sprintf(tail, "%s %s -q", ifile, ofile);
		docmd("MCO.COM");
		erase(ifile);
		strcpy(ifile, ofile); }

	/* Assemble into object module */
	if(oasm) {
		sprintf(ofile,"%s%s.OBJ", link ? temp : "", fnptr);
		message("Assemble... ");
		sprintf(tail,"/t/ml %s,%s;", ifile, ofile);
		docmd("TASM.EXE");

		erase(ifile);
		strcpy(ifile, ofile);

	/* Link into executable program */
		if(link) {
			message(tiny ? "Link TINY...\n" : "Link SMALL...\n");
			sprintf(ofile,"%s.EXE", fnptr);
			sprintf(tail,"/c %s\\PC86RL_%c %s,%s,NUL,%s\\MCLIB;",
				mcdir, tiny ? 'T' : 'S', ifile, ofile, mcdir);
			docmd("TLINK.EXE");
			erase(ifile);
			if(tiny) {
				message("Convert... ");
				sprintf(tail,"%s %s.COM", ofile, fnptr);
				docmd("EXE2BIN.COM");
				erase(ofile); }
			} }

	message("All done.\n");
}

/*
 * Execute a command, looking for it in the MICRO-C directory,
 * and also in any directories found in the PATH environment
 * variable. Operands to the command have been previously
 * defined in the global variable 'tail'.
 */
docmd(char *cmd)
{
	int rc;
	char command[65], path[300], *ptr, *ptr1, c;

	ptr = mcdir;						/* First try MC home dir */
	if(!getenv("PATH", ptr1 = path))	/* And then search  PATH */
		ptr1 = "";

	do {	/* Search MCDIR & PATH for commands */
		sprintf(command,"%s%s%s", ptr, "\\"+(ptr[strlen(ptr)-1] == '\\'), cmd);
		rc = exec(command, tail);
		ptr = ptr1;						/* Point to next directory */
		while(c = *ptr1) {				/* Advance to end of entry */
			++ptr1;
			if(c == ';') {
				*(ptr1 - 1) = 0;		/* Zero terminate */
				break; } } }
	while(((rc == NOFILE) || (rc == NOPATH)) && *ptr);
	if(rc) {
		fprintf(stderr,"%s failed (%d)\n", cmd, rc);
		exit(-1); }
}

/*
 * Output an informational message (verbose mode only)
 */
message(char *ptr)
{
	if(verb)
		fputs(ptr, stderr);
}

/*
 * Erase temporary file (if enabled)
 */
erase(char *file)
{
	if(del)
		delete(file);
}
